# -*- coding: utf-8 -*-
#
# ** header v3.0
# This file is a part of the CaosDB Project.
#
# Copyright (C) 2018 Research Group Biomedical Physics,
# Max-Planck-Institute for Dynamics and Self-Organization Göttingen
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <https://www.gnu.org/licenses/>.
#
# ** end header
#
"""test_server_side_scripting

Integration tests for the implementation of the server-side-scripting api.
"""
from __future__ import print_function, unicode_literals
from pytest import raises, mark
from lxml import etree
from caosdb import get_connection, get_config
from caosdb.exceptions import (EntityDoesNotExistError, ClientErrorException)
from caosdb.connection.encode import MultipartParam, multipart_encode
from caosdb import administration as admin

_TEST_SCRIPTS = ["not_executable", "ok", "err", "simple_script.py"]
_SERVER_SIDE_SCRIPTING_BIN_DIR = get_config().get(
    "IntegrationTests",
    "test_server_side_scripting.bin_dir")
_TEST_SCRIPTS_DIR = "./resources/"
_REMOVE_FILES_AFTERWARDS = []


def setup_module():
    from os import makedirs
    from os.path import join, isdir, exists
    from shutil import copyfile, copymode
    print("bin: " + str(_SERVER_SIDE_SCRIPTING_BIN_DIR))
    print("tests scripts: " + str(_TEST_SCRIPTS))
    if not exists(_SERVER_SIDE_SCRIPTING_BIN_DIR):
        makedirs(_SERVER_SIDE_SCRIPTING_BIN_DIR)
        _REMOVE_FILES_AFTERWARDS.append(_SERVER_SIDE_SCRIPTING_BIN_DIR)
    assert isdir(_SERVER_SIDE_SCRIPTING_BIN_DIR)
    for script_file in _TEST_SCRIPTS:
        target = join(_SERVER_SIDE_SCRIPTING_BIN_DIR, script_file)
        src = join(_TEST_SCRIPTS_DIR, script_file)
        copyfile(src, target)
        copymode(src, target)
        _REMOVE_FILES_AFTERWARDS.append(target)


def teardown_module():
    from os import remove
    from os.path import exists, isdir
    from shutil import rmtree
    for obsolete in _REMOVE_FILES_AFTERWARDS:
        if exists(obsolete):
            if isdir(obsolete):
                rmtree(obsolete)
            else:
                remove(obsolete)


def test_call_script_non_existing():
    form = dict()
    form["call"] = "non_existing_script"
    with raises(EntityDoesNotExistError):
        get_connection().post_form_data("scripting", form)


def test_call_script_not_executable():
    props = admin.get_server_properties()
    print(props)

    import os
    dirpath = os.getcwd()
    print("PWD={}".format(dirpath))
    print("ls ./\n{}".format(os.listdir("./")))
    print("ls ../\n{}".format(os.listdir("../")))

    form = dict()
    form["call"] = "not_executable"
    with raises(ClientErrorException) as exc_info:
        get_connection().post_form_data("scripting", form)
    assert "not executable" in exc_info.value.body.decode("utf-8")


def test_call_ok():
    form = dict()
    form["call"] = "ok"
    r = get_connection().post_form_data("scripting", form)
    xml = etree.parse(r)
    assert xml.xpath("/Response/script/call")[0].text == "ok"
    assert xml.xpath("/Response/script/stdout")[0].text == "ok"
    assert xml.xpath("/Response/script/stderr")[0].text is None
    assert xml.xpath("/Response/script/@code")[0] == "0"


def test_call_err():
    form = dict()
    form["call"] = "err"
    r = get_connection().post_form_data("scripting", form)
    xml = etree.parse(r)
    assert xml.xpath("/Response/script/@code")[0] == "1"
    assert xml.xpath("/Response/script/call")[0].text == "err"
    assert xml.xpath("/Response/script/stdout")[0].text is None
    assert xml.xpath("/Response/script/stderr")[0].text == "err"


@mark.skip(reason="need to setup .pycaosdb.ini in home dirs of sss within docker")
def test_simple_sss():
    form = dict()
    form["call"] = "simple_script.py"
    form["-Oexit"] = "123"
    r = get_connection().post_form_data("scripting", form)
    xml = etree.parse(r)
    print(etree.tostring(xml))
    assert xml.xpath("/Response/script/@code")[0] == "123"

    _REMOVE_FILES_AFTERWARDS.append("test_file.txt")
    with open("test_file.txt", "w") as f:
        f.write("this is a test")
    parts = []
    parts.append(MultipartParam.from_file(paramname="txt_file",
                                          filename="test_file.txt"))
    parts.append(MultipartParam("call", "simple_script.py"))
    body, headers = multipart_encode(parts)
    r = get_connection().insert(["scripting"], body=body, headers=headers)
    xml = etree.parse(r)
    print(etree.tostring(xml).decode("utf-8"))
    assert xml.xpath("/Response/script/@code")[0] == "0"
    assert xml.xpath(
        "/Response/script/call")[0].text.startswith("simple_script.py")
    assert "this is a test" in xml.xpath("/Response/script/stdout")[0].text
    assert xml.xpath("/Response/script/stderr")[0].text is None
